# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: state.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='state.proto',
  package='vss_state',
  serialized_pb=_b('\n\x0bstate.proto\x12\tvss_state\"&\n\x03RGB\x12\t\n\x01r\x18\x01 \x02(\r\x12\t\n\x01g\x18\x02 \x02(\r\x12\t\n\x01\x62\x18\x03 \x02(\r\")\n\x04Pose\x12\t\n\x01x\x18\x01 \x02(\x02\x12\t\n\x01y\x18\x02 \x02(\x02\x12\x0b\n\x03yaw\x18\x03 \x01(\x02\"\x90\x01\n\nBall_State\x12\x1d\n\x04pose\x18\x01 \x02(\x0b\x32\x0f.vss_state.Pose\x12\x1f\n\x06v_pose\x18\x02 \x01(\x0b\x32\x0f.vss_state.Pose\x12\x1f\n\x06k_pose\x18\x03 \x01(\x0b\x32\x0f.vss_state.Pose\x12!\n\x08k_v_pose\x18\x04 \x01(\x0b\x32\x0f.vss_state.Pose\"\xb0\x01\n\x0bRobot_State\x12\x1d\n\x04pose\x18\x01 \x02(\x0b\x32\x0f.vss_state.Pose\x12\x1f\n\x06v_pose\x18\x02 \x01(\x0b\x32\x0f.vss_state.Pose\x12\x1f\n\x06k_pose\x18\x03 \x01(\x0b\x32\x0f.vss_state.Pose\x12!\n\x08k_v_pose\x18\x04 \x01(\x0b\x32\x0f.vss_state.Pose\x12\x1d\n\x05\x63olor\x18\x05 \x01(\x0b\x32\x0e.vss_state.RGB\"\x9f\x02\n\x0cGlobal_State\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0e\n\x06origin\x18\x02 \x01(\x08\x12$\n\x05\x62\x61lls\x18\x03 \x03(\x0b\x32\x15.vss_state.Ball_State\x12-\n\rrobots_yellow\x18\x04 \x03(\x0b\x32\x16.vss_state.Robot_State\x12+\n\x0brobots_blue\x18\x05 \x03(\x0b\x32\x16.vss_state.Robot_State\x12\x11\n\tsituation\x18\x06 \x01(\r\x12\x0c\n\x04time\x18\x07 \x01(\r\x12\x14\n\x0cgoals_yellow\x18\x08 \x01(\r\x12\x12\n\ngoals_blue\x18\t \x01(\r\x12\x13\n\x0bname_yellow\x18\n \x01(\t\x12\x11\n\tname_blue\x18\x0b \x01(\t')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_RGB = _descriptor.Descriptor(
  name='RGB',
  full_name='vss_state.RGB',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='r', full_name='vss_state.RGB.r', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='g', full_name='vss_state.RGB.g', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='b', full_name='vss_state.RGB.b', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=64,
)


_POSE = _descriptor.Descriptor(
  name='Pose',
  full_name='vss_state.Pose',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='vss_state.Pose.x', index=0,
      number=1, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='y', full_name='vss_state.Pose.y', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='yaw', full_name='vss_state.Pose.yaw', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=107,
)


_BALL_STATE = _descriptor.Descriptor(
  name='Ball_State',
  full_name='vss_state.Ball_State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pose', full_name='vss_state.Ball_State.pose', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='v_pose', full_name='vss_state.Ball_State.v_pose', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='k_pose', full_name='vss_state.Ball_State.k_pose', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='k_v_pose', full_name='vss_state.Ball_State.k_v_pose', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=110,
  serialized_end=254,
)


_ROBOT_STATE = _descriptor.Descriptor(
  name='Robot_State',
  full_name='vss_state.Robot_State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pose', full_name='vss_state.Robot_State.pose', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='v_pose', full_name='vss_state.Robot_State.v_pose', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='k_pose', full_name='vss_state.Robot_State.k_pose', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='k_v_pose', full_name='vss_state.Robot_State.k_v_pose', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='color', full_name='vss_state.Robot_State.color', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=257,
  serialized_end=433,
)


_GLOBAL_STATE = _descriptor.Descriptor(
  name='Global_State',
  full_name='vss_state.Global_State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='vss_state.Global_State.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='origin', full_name='vss_state.Global_State.origin', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='balls', full_name='vss_state.Global_State.balls', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='robots_yellow', full_name='vss_state.Global_State.robots_yellow', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='robots_blue', full_name='vss_state.Global_State.robots_blue', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='situation', full_name='vss_state.Global_State.situation', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time', full_name='vss_state.Global_State.time', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='goals_yellow', full_name='vss_state.Global_State.goals_yellow', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='goals_blue', full_name='vss_state.Global_State.goals_blue', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name_yellow', full_name='vss_state.Global_State.name_yellow', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name_blue', full_name='vss_state.Global_State.name_blue', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=436,
  serialized_end=723,
)

_BALL_STATE.fields_by_name['pose'].message_type = _POSE
_BALL_STATE.fields_by_name['v_pose'].message_type = _POSE
_BALL_STATE.fields_by_name['k_pose'].message_type = _POSE
_BALL_STATE.fields_by_name['k_v_pose'].message_type = _POSE
_ROBOT_STATE.fields_by_name['pose'].message_type = _POSE
_ROBOT_STATE.fields_by_name['v_pose'].message_type = _POSE
_ROBOT_STATE.fields_by_name['k_pose'].message_type = _POSE
_ROBOT_STATE.fields_by_name['k_v_pose'].message_type = _POSE
_ROBOT_STATE.fields_by_name['color'].message_type = _RGB
_GLOBAL_STATE.fields_by_name['balls'].message_type = _BALL_STATE
_GLOBAL_STATE.fields_by_name['robots_yellow'].message_type = _ROBOT_STATE
_GLOBAL_STATE.fields_by_name['robots_blue'].message_type = _ROBOT_STATE
DESCRIPTOR.message_types_by_name['RGB'] = _RGB
DESCRIPTOR.message_types_by_name['Pose'] = _POSE
DESCRIPTOR.message_types_by_name['Ball_State'] = _BALL_STATE
DESCRIPTOR.message_types_by_name['Robot_State'] = _ROBOT_STATE
DESCRIPTOR.message_types_by_name['Global_State'] = _GLOBAL_STATE

RGB = _reflection.GeneratedProtocolMessageType('RGB', (_message.Message,), dict(
  DESCRIPTOR = _RGB,
  __module__ = 'state_pb2'
  # @@protoc_insertion_point(class_scope:vss_state.RGB)
  ))
_sym_db.RegisterMessage(RGB)

Pose = _reflection.GeneratedProtocolMessageType('Pose', (_message.Message,), dict(
  DESCRIPTOR = _POSE,
  __module__ = 'state_pb2'
  # @@protoc_insertion_point(class_scope:vss_state.Pose)
  ))
_sym_db.RegisterMessage(Pose)

Ball_State = _reflection.GeneratedProtocolMessageType('Ball_State', (_message.Message,), dict(
  DESCRIPTOR = _BALL_STATE,
  __module__ = 'state_pb2'
  # @@protoc_insertion_point(class_scope:vss_state.Ball_State)
  ))
_sym_db.RegisterMessage(Ball_State)

Robot_State = _reflection.GeneratedProtocolMessageType('Robot_State', (_message.Message,), dict(
  DESCRIPTOR = _ROBOT_STATE,
  __module__ = 'state_pb2'
  # @@protoc_insertion_point(class_scope:vss_state.Robot_State)
  ))
_sym_db.RegisterMessage(Robot_State)

Global_State = _reflection.GeneratedProtocolMessageType('Global_State', (_message.Message,), dict(
  DESCRIPTOR = _GLOBAL_STATE,
  __module__ = 'state_pb2'
  # @@protoc_insertion_point(class_scope:vss_state.Global_State)
  ))
_sym_db.RegisterMessage(Global_State)


# @@protoc_insertion_point(module_scope)
