syntax = "proto3";

import "command_fira.proto";
import "replacement.proto";
import "common.proto";

package fira_message.sim_to_ref;

message Packet {
	Commands    cmd     = 1;
	Replacement replace = 2;
}

message Environment {
	uint32 step = 1;
	Frame frame = 2;
	Field field = 3;
	uint32 goals_blue = 4;
	uint32 goals_yellow =5;
}

service Simulate {
	rpc Simulate (Packet) returns (Environment) {}
}
