# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: command.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='command.proto',
  package='fira_message.sim_to_ref',
  syntax='proto3',
  serialized_pb=_b('\n\rcommand.proto\x12\x17\x66ira_message.sim_to_ref\"R\n\x07\x43ommand\x12\n\n\x02id\x18\x01 \x01(\r\x12\x12\n\nyellowteam\x18\x02 \x01(\x08\x12\x12\n\nwheel_left\x18\x06 \x01(\x01\x12\x13\n\x0bwheel_right\x18\x07 \x01(\x01\"D\n\x08\x43ommands\x12\x38\n\x0erobot_commands\x18\x01 \x03(\x0b\x32 .fira_message.sim_to_ref.Commandb\x06proto3')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_COMMAND = _descriptor.Descriptor(
  name='Command',
  full_name='fira_message.sim_to_ref.Command',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='fira_message.sim_to_ref.Command.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='yellowteam', full_name='fira_message.sim_to_ref.Command.yellowteam', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='wheel_left', full_name='fira_message.sim_to_ref.Command.wheel_left', index=2,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='wheel_right', full_name='fira_message.sim_to_ref.Command.wheel_right', index=3,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=42,
  serialized_end=124,
)


_COMMANDS = _descriptor.Descriptor(
  name='Commands',
  full_name='fira_message.sim_to_ref.Commands',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='robot_commands', full_name='fira_message.sim_to_ref.Commands.robot_commands', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=126,
  serialized_end=194,
)

_COMMANDS.fields_by_name['robot_commands'].message_type = _COMMAND
DESCRIPTOR.message_types_by_name['Command'] = _COMMAND
DESCRIPTOR.message_types_by_name['Commands'] = _COMMANDS

Command = _reflection.GeneratedProtocolMessageType('Command', (_message.Message,), dict(
  DESCRIPTOR = _COMMAND,
  __module__ = 'command_pb2'
  # @@protoc_insertion_point(class_scope:fira_message.sim_to_ref.Command)
  ))
_sym_db.RegisterMessage(Command)

Commands = _reflection.GeneratedProtocolMessageType('Commands', (_message.Message,), dict(
  DESCRIPTOR = _COMMANDS,
  __module__ = 'command_pb2'
  # @@protoc_insertion_point(class_scope:fira_message.sim_to_ref.Commands)
  ))
_sym_db.RegisterMessage(Commands)


# @@protoc_insertion_point(module_scope)
